#!/bin/bash
# ------------------------------------------------------------------
# llxcfg-vars.sh
#
# variable manipulation functions for LliureX
# (c) Luis Garcia <garcia_luigis@gva.es> for LliureX (May 2006)
#
# Some code was taken from previous versions of llxcfg-showvars
# originally written by Sergio Talens-Oliag <sto@gva.es>
# ------------------------------------------------------------------
#

set -e

# VARIABLES
LLX_COMMONLIB="/usr/share/lliurex/llxcfg-common/llxcfg-common.sh"
[ -f "${LLX_COMMONLIB}" ] || exit 1
. ${LLX_COMMONLIB}

CONFFILE="/etc/llxcfg/llxcfg.conf"
CONFDIRS="$LLXVARS_DIRS"
SCRIPTS_DIR="/usr/share/llxcfg/scripts.d"
VAR_REGEXP="^[[:blank:]]*[[:alpha:]_][[:alnum:]_]*[[:blank:]]*="
VARS_FILENAME="10_llxcfg_setvars"

# Read system override
if [ -f "$CONFFILE" ]; then
  source "$CONFFILE"
fi

exit_message(){
   while [ "$1" ] ; do
      echo "$1" >&2
      shift
   done
   exit 1
}

llxcfg_vars_test_CONFDIRS(){
   # Comprobamos la existencia y el contenido de la variable CONFDIRS
   [ "${CONFDIRS}" ] || return 1
   for d in ${CONFDIRS}; do
      [ -d ${d} ] || return 1
   done
   return 0
}
		     

llxcfg_vars_files0(){
   local d
   for d in $CONFDIRS; do
      find ${d}/ -mindepth 1 -maxdepth 1 -xtype f
   done
}

llxcfg_vars_files(){
   local f
   llxcfg_vars_files0 2>/dev/null | while read f ; do
      if file "$f" |grep -q ".* text$" ; then
         echo "$f"
      fi
   done
   return 0
}

llxcfg_vars_files2(){
   local d
   for d in $CONFDIRS; do
      find ${d}/ -mindepth 1 -maxdepth 1 -xtype f -exec sh -c 'file "{}"|grep -q ".* text$" && echo {}' \;
   done
}

llxcfg_vars_files3(){
   file `llxcfg_vars_files0` |sed -ne "/text$/s%:.*%%p"
}

llxcfg_vars_lines(){
   # Print the lines we want
   # -----------------------
   # 1. grep gets the lines of the form 'VAR=VALUE"
   # 2. sed removes whitespace, double quotes around the value and all the 
   #    quotes and finally it puts puts quotes around the given value.
   local f VARFILES="$@"
   [ -z "$VARFILES" ] && VARFILES="`llxcfg_vars_files`"
   [ -z "$VARFILES" ] && return 0
   
   cat $VARFILES 2>/dev/null \
| grep "${VAR_REGEXP}" \
| sed -e " \
    s%^[[:blank:]]*%%;\
    s%[[:blank:]]*$%%;\
    s%[[:blank:]]*=[[:blank:]]*%=%;\
    s%=\"\(.*\)\"[[:blank:]]*;\?$%=\1%;\
    s%='\(.*\)'[[:blank:]]*;\?$%=\1%;\
    s%=\`\(.*\)\`[[:blank:]]*;\?$%=\1%;\
    s%'%%g;\
    s%=\(.*\)$%='\1';%;\
"
   return 0
}

llxcfg_vars_names(){
   llxcfg_vars_lines $@ |sed -e "s%=.*$%%" |sort -u
}

llxcfg_vars_info(){
   local lastf f v vlista VARNAMES VARFILES="`llxcfg_vars_files`"
   [ -z "$VARFILES" ] && return 0
   if [ $# -lt 1 ]; then
      VARNAMES="`llxcfg_vars_names`"
   else
      VARNAMES="$@"
   fi
   [ -z "$VARNAMES" ] && return 0
   for v in $VARNAMES ; do
      echo "$v:"
      lastf=""
      vlista=$(grep -l "^[[:space:]]*${v}[[:space:]]*=" $VARFILES || true)
      for f in $vlista; do
         [ "$lastf" ] && echo -e "\t[`llxcfg_vars_show ${lastf} $v` ($lastf)]"
	 lastf=$f
      done
      [ "$lastf" ] && echo -e "\t `llxcfg_vars_show ${lastf} $v` ($lastf)\n"
   done
 
}

llxcfg_vars_show(){
   # ... llxcfg_vars_lines and ...
   # 3. awk prints the values of the requested variables
   
   local VARFILES="$1"
   shift
   
   llxcfg_vars_lines ${VARFILES} | awk -F'=' -- \
'BEGIN { for(i=1;i<ARGC;i++) { k[ARGV[i]]=""; delete ARGV[i];} } \
{ if ($1 in k) a[$1]=$0; } \
END { for (v in k) if (v in a) print a[v]; }' $@
}

llxcfg_vars_action(){
   # admin/system
   VARS_MODE="$1"
   #remove default dir
   AVAILABLE_DIRS="${LLXVARS_DIRS#* }"
   [ "$VARS_MODE" = "admin" ] && AVAILABLE_DIRS="${AVAILABLE_DIRS#* }"
   VARS_DIR="${AVAILABLE_DIRS%% *}"
   VARS_FILE="${VARS_DIR}/${VARS_FILENAME}"

   # add/del unset
   ACTION="$2"
   shift 2
   SED_CMD=""
   VAR_VALUE=""
   TEMP_FILE="`tempfile`"
   while [ "$1" ] ; do
      PARAM="$1"
      shift
      VAR_NAME=""
      if [ "$ACTION" = "add" ] ; then
         if echo "${PARAM}" |grep -q "${VAR_REGEXP}" ; then
            VAR_NAME=${PARAM%%=*}
            VAR_VALUE=${PARAM#*=}
         fi
      else
         echo "${PARAM}" |grep -q "[[:alpha:]_][[:alnum:]_]*" && VAR_NAME="${PARAM}"
      fi
      if [ "$VAR_NAME" ] ; then
         [ "$ACTION" != "del" ] && echo "${VAR_NAME}=${VAR_VALUE}" >> $TEMP_FILE
         SED_CMD="$SED_CMD;/^[[:blank:]]*${VAR_NAME}[[:blank:]]*=/d"
      fi
   done
   if [ "$SED_CMD" ] ; then
      [ -e "${VARS_FILE}" ] || echo "# llxcfg-setvars variable file" |llxcfg-install --mode=${LLXCFG_MODE} --group=${LLXCFG_GROUP} - "${VARS_FILE}"
      ( sed -e "$SED_CMD" "${VARS_FILE}" ; cat $TEMP_FILE ) |llxcfg-install --mode=${LLXCFG_MODE} --group=${LLXCFG_GROUP} - "${VARS_FILE}"
   fi
   rm -f $TEMP_FILE
   return 0
}

