#!/bin/bash
# -------
# File:        llxcfg-cpkg.sh
# Description: LliureX Config Packages common variables and functions
# Author:      Luis Garcia Gisbert <garcia_luigis@gva.es>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along 
# with this program; if not, write to the Free Software Foundation, Inc., 
# 51 Franklin St, Fifth Floor, Boston MA 02110-1301 USA
# --------

# library
LLX_COMMONLIB="/usr/share/lliurex/llxcfg-common/llxcfg-common.sh"
[ -f "${LLX_COMMONLIB}" ] || exit 1
. ${LLX_COMMONLIB}
   

# variables
declare -a PKG_LIST PKG_RC

CPKG_BACKEND_ACTIONS="pre-dpkg post-dpkg update revert configure deconfigure postconfigure enable disable allow deny invoke"

CPKG_RC_NOTROOT=100
CPKG_RC_BADPARAMETER=150
CPKG_RC_NOTRUNNINGDAEMON=200

# functions

is_backend_option(){
   echo " $CPKG_BACKEND_ACTIONS " |grep -q " $1 " && return 0
   return 1
}

error_msg() {
  say_msg "Error: " "$@"  >&2
}

warn_msg() {
  say_msg "Warning: " "$@" >&2
}

say_msg() {
#  [ -z "$VERBOSE" ] && return 0
  if [ $# -gt 1 ] ; then
     PRE_MSG="$1"
     PRE_BLANK=""
     j=0
     while [ ${j} -lt ${#PRE_MSG} ] ;do 
        PRE_BLANK="${PRE_BLANK} "
        j=$(($j + 1 ))
     done
     shift
  else
     PRE_MSG=""
     PRE_BLANK=""
  fi 
  while [ "$1" ] ; do
     [ -z "$VERBOSE" ] || echo -e "${PRE_MSG}${1}"
     echo -e "${PRE_MSG}${1}" | logger -t "llxcfg-cpkg[$$]"
     PRE_MSG="${PRE_BLANK}"
     shift
  done
  return 0
}

get_all_pkg(){
    PLIST=""
    find  "${CPKG_ENABLED}" -type l |sort |while read line; do
      PNAME="$(basename $(readlink -f ${line}))"
      if ! echo " $PLIST " | grep -q " ${PNAME} " ; then
         echo "${PNAME}"
         PLIST="$PLIST $PNAME"
      fi
    done
    return 0
}

get_pkg_aliases(){
   echo "$1"
   [ ! -s "${CPKG_ORIGINAL}/$1/$CPKG_ALIAS_FILE" ] || cat "${CPKG_ORIGINAL}/$1/$CPKG_ALIAS_FILE" |sed -e "/^[[:blank:]]*$/d"
} 

get_pkg_list(){
   # build package list from (optional) parameters
   # or from full package list (default) but
   # remove disabled packages
   REQUIRED_PKG="$@"
   if [ "$PKG_FILE" -a -z "$REQUIRED_PKG" ] ; then
      REQUIRED_PKG="$(get_all_pkg |tr "\n" " ")"
   fi
   PKG_TMP="$(tempfile)"
   # "find" CPKG_UPDATE_DIR files asociated to pkg list and concatenate (cat) in a single list
   if [ "$PKG_FILE" -a -s "$PKG_FILE" ] ; then
      cat $(sed -e "s%^%$CPKG_UPDATE_DIR/%;s%[[:blank:]]\+.*$%%" "$PKG_FILE") 2>/dev/null |sed -e "/^[[:blank:]]*$/d" > "$PKG_TMP"
   fi
   if [ "$REQUIRED_PKG" ] ; then
      # verify required pkgs and sort them in correct order
      REQUIRED_PKG=" $REQUIRED_PKG "
      ALL_PKG=($(get_all_pkg))
      i=0
      n=0
      while [ $i -lt ${#ALL_PKG[@]} ] ; do
         CURRENT_P="${ALL_PKG[$i]}"
         i=$(( $i + 1 ))
         ADD_CURRENT=""
         for P in $(get_pkg_aliases "$CURRENT_P") ; do
            if echo "$REQUIRED_PKG" |grep -q " $P " ; then
               ADD_CURRENT="Y"
               break
            fi
         done
         CURRENT_F="${CPKG_ORIGINAL}/$CURRENT_P/$CPKG_PKGS_FILE"
         if [ "$ADD_CURRENT" -a "$PKG_FILE" -a -s "$PKG_FILE" ] ; then
            if [ -s "$CURRENT_F" ] ; then
               ADD_CURRENT=""
               if sed -e "s%[[:blank:]]\+.*$%%" "$PKG_FILE" |grep -qFxf "$CURRENT_F" ; then
                  ADD_CURRENT="Y"
               fi
            fi
            if [ -z "$ADD_CURRENT" -a -s "$PKG_TMP" ] && get_pkg_aliases "$CURRENT_P" |grep -qFxf - "$PKG_TMP" ; then
               ADD_CURRENT="Y"
            fi
         fi
         if [ "$ADD_CURRENT" ] ; then
            PKG_LIST[$n]="$CURRENT_P"
            n=$(( $n + 1 ))
         fi
      done
   else
      PKG_LIST=($(get_all_pkg))
   fi
   rm -f "$PKG_TMP"
   if [ ${#PKG_LIST[@]} -eq 0 ] ; then
      return 1
   fi
   return 0
}

pkg_info(){
   # control file
   p="$1"
   CTL_FILE="${CPKG_ORIGINAL}/${p}/${CPKG_CONTROL_FILE}"
   [ ! -r "$CTL_FILE" ] || sed -e "s%^%${p}:%" "$CTL_FILE"
   # services
   SERVICES=""
   SRV_FILE="${CPKG_ORIGINAL}/${p}/${CPKG_SERVICES_FILE}"
   [ ! -s "$SRV_FILE" ] || SERVICES="$(cat "$SRV_FILE"|tr "\n" ,)"
   [ -z "$SERVICES" ] || echo "${p}:Services: ${SERVICES}" |sed -e "s%,%, %g;s%,[[:space:]]*$%%g"
   # packages
   PKGSL=""
   PKGS_FILE="${CPKG_ORIGINAL}/${p}/$CPKG_PKGS_FILE"
   [ ! -s "$PKGS_FILE" ] || PKGSL="$(cat "$PKGS_FILE"|tr "\n" ,)"
   [ -z "$PKGSL" ] || echo "${p}:Packages: ${PKGSL}" |sed -e "s%,%, %g;s%,[[:space:]]*$%%g"
   # Alias
   ALIASES=""
   ALIAS_FILE="${CPKG_ORIGINAL}/${p}/$CPKG_ALIAS_FILE"
   [ ! -s "$ALIAS_FILE" ] || ALIASES="$(cat "$ALIAS_FILE"|tr "\n" ,)"
   [ -z "$ALIASES" ] || echo "${p}:Alias: ${ALIASES}" |sed -e "s%,%, %g;s%,[[:space:]]*$%%g"
   return 0
}

pkg_exists(){
   [ -d "${CPKG_ORIGINAL}/${1}" ] && return 0
   warn_msg "Skipping unavailable config package \"$1\""
   return 1
}

is_pkg_enabled(){
   [ "$(find "${CPKG_ENABLED}" -lname "$(cpkg_reldir "$1")" 2>/dev/null)" ] && return 0
   warn_msg "Skipping disabled config package \"$1\""
   return ${RC_NOTFOUND}
}

is_pkg_allowed(){
   if rgrep -q "^${1}$" "${CPKG_DENIED}" 2>/dev/null ; then
      warn_msg "Skipping denied config package \"$1\""
      return ${RC_DISABLED}
   fi   
   return 0
}

test_pkg(){
   pkg_exists "$1" || return $?
   is_pkg_enabled "$1" || return $?
   is_pkg_allowed "$1" || return $?
   return 0
}

